@@ -1,4 +1,4 @@
-#!/usr/bin/python
+#!/usr/bin/env python3
 #
 # Copyright 2010 Google Inc.
 #
@@ -20,32 +20,35 @@ Shows their status graphically.
 """
 
 __author__ = 'Scott Kirkwood (scott+keymon@forusers.com)'
-__version__ = '1.17'
+__version__ = '1.20'
 
+import gettext
 import locale
 import logging
-import pygtk
-pygtk.require('2.0')
-import gettext
-import gobject
-import gtk
 import os
 import sys
 import time
+
 try:
-  import xlib
+  from . import xlib
 except ImportError:
-  print 'Error: Missing xlib, run sudo apt-get install python-xlib'
+  print('Error: Missing xlib, run sudo apt-get install python3-xlib')
   sys.exit(-1)
 
-import options
-import lazy_pixbuf_creator
-import mod_mapper
-import settings
-import shaped_window
-import two_state_image
+from . import options
+from . import lazy_pixbuf_creator
+from . import mod_mapper
+from . import settings
+from . import shaped_window
+from . import two_state_image
 
-from ConfigParser import SafeConfigParser
+import cairo
+import gi
+gi.require_version("Gtk", "3.0")
+gi.require_foreign("cairo")
+from gi.repository import Gtk, GdkPixbuf, GObject, GLib
+
+from configparser import SafeConfigParser
 
 gettext.install('key-mon', 'locale')
 
@@ -59,14 +62,14 @@ def fix_svg_key_closure(fname, from_tos):
 
   def fix_svg_key():
     """Given an SVG file return the SVG text fixed."""
-    logging.debug('Read file %r', fname)
+    logging.debug('Read file %s', fname)
     fin = open(fname)
     fbytes = fin.read()
     fin.close()
-    for fin, t in from_tos:
+    for fin, txt in from_tos:
       # Quick XML escape fix
-      t = t.replace('<', '&lt;')
-      fbytes = fbytes.replace(fin, t)
+      txt = txt.replace('<', '&lt;')
+      fbytes = fbytes.replace(fin, txt)
     return fbytes
 
   return fix_svg_key
@@ -74,17 +77,17 @@ def fix_svg_key_closure(fname, from_tos):
 
 def cstrf(func):
   """Change locale before using str function"""
-  OLD_CTYPE = locale.getlocale(locale.LC_CTYPE)
+  old_ctype = locale.getlocale(locale.LC_CTYPE)
   locale.setlocale(locale.LC_CTYPE, 'C')
-  s = func()
-  locale.setlocale(locale.LC_CTYPE, OLD_CTYPE)
-  return s
+  func_result = func()
+  locale.setlocale(locale.LC_CTYPE, old_ctype)
+  return func_result
 
 
 class KeyMon:
   """main KeyMon window class."""
 
-  def __init__(self, options):
+  def __init__(self, opts):
     """Create the Key Mon window.
     Options dict:
       scale: float 1.0 is default which means normal size.
@@ -97,7 +100,7 @@ class KeyMon:
     self.btns = ['MOUSE', 'BTN_RIGHT', 'BTN_MIDDLE', 'BTN_MIDDLERIGHT',
                  'BTN_LEFT', 'BTN_LEFTRIGHT', 'BTN_LEFTMIDDLE',
                  'BTN_LEFTMIDDLERIGHT']
-    self.options = options
+    self.options = opts
     self.pathname = os.path.dirname(os.path.abspath(__file__))
     if self.options.scale < 1.0:
       self.svg_size = '-small'
@@ -114,13 +117,11 @@ class KeyMon:
     self.no_press_timer = None
 
     self.move_dragged = False
-    self.shape_mask_current = None
-    self.shape_mask_cache = {}
 
-    self.MODS = ['SHIFT', 'CTRL', 'META', 'ALT']
-    self.IMAGES = ['MOUSE'] + self.MODS
-    self.images = dict([(img, None) for img in self.IMAGES])
-    self.enabled = dict([(img, self.get_option(cstrf(img.lower))) for img in self.IMAGES])
+    self.mod_constants = ['SHIFT', 'CTRL', 'META', 'ALT']
+    self.images_constants = ['MOUSE'] + self.mod_constants
+    self.images = {img: None for img in self.images_constants}
+    self.enabled = {img: self.get_option(cstrf(img.lower)) for img in self.images_constants}
 
 
     self.options.kbd_files = settings.get_kbd_files()
@@ -148,7 +149,7 @@ class KeyMon:
         if key.startswith('KEY_'):
           key_info = self.modmap.get_from_name(key)
           if not key_info:
-            print 'Key %s not found' % key
+            print(f'Key {key} not found')
             self.destroy(None)
             return
           scancode = key_info[0]
@@ -157,25 +158,23 @@ class KeyMon:
           event = xlib.XEvent('EV_KEY', scancode=0, code=key, value=1)
 
         self.handle_event(event)
-        while gtk.events_pending():
-          gtk.main_iteration(False)
+        while Gtk.events_pending():
+          Gtk.main_iteration(False)
         time.sleep(0.1)
-      except Exception, exp:
-        print exp
-    while gtk.events_pending():
-      gtk.main_iteration(False)
+      except Exception as exp:
+        print(exp)
+    while Gtk.events_pending():
+      Gtk.main_iteration()
     time.sleep(0.1)
     win = self.window
     x, y = win.get_position()
     w, h = win.get_size()
-    screenshot = gtk.gdk.Pixbuf.get_from_drawable(
-        gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB, True, 8, w, h),
-        gtk.gdk.get_default_root_window(),
-        gtk.gdk.colormap_get_system(),
-        x, y, 0, 0, w, h)
+
+    screenshot = GdkPixbuf.Pixbuf.new_from_resource(win) # TODO(scott): not working
+
     fname = 'screenshot.png'
-    screenshot.save(fname, 'png')
-    print 'Saved screenshot %r' % fname
+    screenshot.savev(fname, 'png')
+    print(f'Saved screenshot {fname!r}')
     self.destroy(None)
 
   def create_names_to_fnames(self):
@@ -185,32 +184,32 @@ class KeyMon:
     else:
       self.svg_size = ''
     ftn = {
-      'MOUSE': [self.svg_name('mouse'),],
-      'BTN_MIDDLE': [self.svg_name('mouse'), self.svg_name('middle-mouse')],
-      'SCROLL_UP': [self.svg_name('mouse'), self.svg_name('scroll-up-mouse')],
-      'SCROLL_DOWN': [self.svg_name('mouse'), self.svg_name('scroll-dn-mouse')],
+        'MOUSE': [self.svg_name('mouse'),],
+        'BTN_MIDDLE': [self.svg_name('mouse'), self.svg_name('middle-mouse')],
+        'SCROLL_UP': [self.svg_name('mouse'), self.svg_name('scroll-up-mouse')],
+        'SCROLL_DOWN': [self.svg_name('mouse'), self.svg_name('scroll-dn-mouse')],
 
-      'REL_LEFT': [self.svg_name('mouse'), self.svg_name('sroll-lft-mouse')],
-      'REL_RIGHT': [self.svg_name('mouse'), self.svg_name('scroll-rgt-mouse')],
-      'SHIFT': [self.svg_name('shift')],
-      'SHIFT_EMPTY': [self.svg_name('shift'), self.svg_name('whiteout-72')],
-      'CTRL': [self.svg_name('ctrl')],
-      'CTRL_EMPTY': [self.svg_name('ctrl'), self.svg_name('whiteout-58')],
-      'META': [self.svg_name('meta'), self.svg_name('meta')],
-      'META_EMPTY': [self.svg_name('meta'), self.svg_name('whiteout-58')],
-      'ALT': [self.svg_name('alt')],
-      'ALT_EMPTY': [self.svg_name('alt'), self.svg_name('whiteout-58')],
-      'ALTGR': [self.svg_name('altgr')],
-      'ALTGR_EMPTY': [self.svg_name('altgr'), self.svg_name('whiteout-58')],
-      'KEY_EMPTY': [
-          fix_svg_key_closure(self.svg_name('one-char-template'), [('&amp;', '')]),
-              self.svg_name('whiteout-48')],
-      'BTN_LEFTRIGHT': [
-          self.svg_name('mouse'), self.svg_name('left-mouse'),
-          self.svg_name('right-mouse')],
-      'BTN_LEFTMIDDLERIGHT': [
-          self.svg_name('mouse'), self.svg_name('left-mouse'),
-          self.svg_name('middle-mouse'), self.svg_name('right-mouse')],
+        'REL_LEFT': [self.svg_name('mouse'), self.svg_name('sroll-lft-mouse')],
+        'REL_RIGHT': [self.svg_name('mouse'), self.svg_name('scroll-rgt-mouse')],
+        'SHIFT': [self.svg_name('shift')],
+        'SHIFT_EMPTY': [self.svg_name('shift'), self.svg_name('whiteout-72')],
+        'CTRL': [self.svg_name('ctrl')],
+        'CTRL_EMPTY': [self.svg_name('ctrl'), self.svg_name('whiteout-58')],
+        'META': [self.svg_name('meta'), self.svg_name('meta')],
+        'META_EMPTY': [self.svg_name('meta'), self.svg_name('whiteout-58')],
+        'ALT': [self.svg_name('alt')],
+        'ALT_EMPTY': [self.svg_name('alt'), self.svg_name('whiteout-58')],
+        'ALTGR': [self.svg_name('altgr')],
+        'ALTGR_EMPTY': [self.svg_name('altgr'), self.svg_name('whiteout-58')],
+        'KEY_EMPTY': [
+            fix_svg_key_closure(self.svg_name('one-char-template'), [('&amp;', '')]),
+            self.svg_name('whiteout-48')],
+        'BTN_LEFTRIGHT': [
+            self.svg_name('mouse'), self.svg_name('left-mouse'),
+            self.svg_name('right-mouse')],
+        'BTN_LEFTMIDDLERIGHT': [
+            self.svg_name('mouse'), self.svg_name('left-mouse'),
+            self.svg_name('middle-mouse'), self.svg_name('right-mouse')],
     }
     if self.options.swap_buttons:
       # swap the meaning of left and right
@@ -221,59 +220,87 @@ class KeyMon:
       right_str = 'right'
 
     ftn.update({
-      'BTN_RIGHT': [self.svg_name('mouse'),
-        self.svg_name('%s-mouse' % right_str)],
-      'BTN_LEFT': [self.svg_name('mouse'),
-        self.svg_name('%s-mouse' % left_str)],
-      'BTN_LEFTMIDDLE': [
-          self.svg_name('mouse'), self.svg_name('%s-mouse' % left_str),
-          self.svg_name('middle-mouse')],
-      'BTN_MIDDLERIGHT': [
-          self.svg_name('mouse'), self.svg_name('middle-mouse'),
-          self.svg_name('%s-mouse' % right_str)],
+        'BTN_RIGHT': [
+            self.svg_name('mouse'),
+            self.svg_name(f'{right_str}-mouse')],
+        'BTN_LEFT': [
+            self.svg_name('mouse'),
+            self.svg_name(f'{left_str}-mouse')],
+        'BTN_LEFTMIDDLE': [
+            self.svg_name('mouse'), self.svg_name(f'{left_str}-mouse'),
+            self.svg_name('middle-mouse')],
+        'BTN_MIDDLERIGHT': [
+            self.svg_name('mouse'), self.svg_name('middle-mouse'),
+            self.svg_name(f'{right_str}-mouse')],
     })
 
     if self.options.scale >= 1.0:
       ftn.update({
-        'KEY_SPACE': [
-            fix_svg_key_closure(self.svg_name('two-line-wide'),
-            [('TOP', 'Space'), ('BOTTOM', '')])],
-        'KEY_TAB': [
-            fix_svg_key_closure(self.svg_name('two-line-wide'),
-            [('TOP', 'Tab'), ('BOTTOM', u'\u21B9')])],
-        'KEY_BACKSPACE': [
-            fix_svg_key_closure(self.svg_name('two-line-wide'),
-            [('TOP', 'Back'), ('BOTTOM', u'\u21fd')])],
-        'KEY_RETURN': [
-            fix_svg_key_closure(self.svg_name('two-line-wide'),
-            [('TOP', 'Enter'), ('BOTTOM', u'\u23CE')])],
-        'KEY_CAPS_LOCK': [
-            fix_svg_key_closure(self.svg_name('two-line-wide'),
-            [('TOP', 'Capslock'), ('BOTTOM', '')])],
-        'KEY_MULTI_KEY': [
-            fix_svg_key_closure(self.svg_name('two-line-wide'),
-            [('TOP', 'Compose'), ('BOTTOM', '')])],
+          'KEY_SPACE': [
+              fix_svg_key_closure(
+                  self.svg_name('two-line-wide'),
+                  [('TOP', 'Space'), ('BOTTOM', '')])],
+          'KEY_TAB': [
+              fix_svg_key_closure(
+                  self.svg_name('two-line-wide'),
+                  [('TOP', 'Tab'),
+                   ('BOTTOM', '\N{Leftwards arrow to bar over rightwards arrow to bar}')])],
+          'KEY_BACKSPACE': [
+              fix_svg_key_closure(
+                  self.svg_name('two-line-wide'),
+                  [('TOP', 'Back'), ('BOTTOM', '\N{Leftwards open-headed arrow}')])],
+          'KEY_RETURN': [
+              fix_svg_key_closure(
+                  self.svg_name('two-line-wide'),
+                  [('TOP', 'Enter'), ('BOTTOM', '\N{Return symbol}')])],
+          'KEY_CAPS_LOCK': [
+              fix_svg_key_closure(
+                  self.svg_name('two-line-wide'),
+                  [('TOP', 'Capslock'), ('BOTTOM', '')])],
+          'KEY_MULTI_KEY': [
+              fix_svg_key_closure(
+                  self.svg_name('two-line-wide'),
+                  [('TOP', 'Compose'), ('BOTTOM', '')])],
       })
     else:
       ftn.update({
-        'KEY_SPACE': [
-            fix_svg_key_closure(self.svg_name('one-line-wide'), [('&amp;', 'Space')])],
-        'KEY_TAB': [
-            fix_svg_key_closure(self.svg_name('one-line-wide'), [('&amp;', 'Tab')])],
-        'KEY_BACKSPACE': [
-            fix_svg_key_closure(self.svg_name('one-line-wide'), [('&amp;', 'Back')])],
-        'KEY_RETURN': [
-            fix_svg_key_closure(self.svg_name('one-line-wide'), [('&amp;', 'Enter')])],
-        'KEY_CAPS_LOCK': [
-            fix_svg_key_closure(self.svg_name('one-line-wide'), [('&amp;', 'Capslck')])],
-        'KEY_MULTI_KEY': [
-            fix_svg_key_closure(self.svg_name('one-line-wide'), [('&amp;', 'Compose')])],
+          'KEY_SPACE': [
+              fix_svg_key_closure(self.svg_name('one-line-wide'), [('&amp;', 'Space')])],
+          'KEY_TAB': [
+              fix_svg_key_closure(self.svg_name('one-line-wide'), [('&amp;', 'Tab')])],
+          'KEY_BACKSPACE': [
+              fix_svg_key_closure(self.svg_name('one-line-wide'), [('&amp;', 'Back')])],
+          'KEY_RETURN': [
+              fix_svg_key_closure(self.svg_name('one-line-wide'), [('&amp;', 'Enter')])],
+          'KEY_CAPS_LOCK': [
+              fix_svg_key_closure(self.svg_name('one-line-wide'), [('&amp;', 'Capslck')])],
+          'KEY_MULTI_KEY': [
+              fix_svg_key_closure(self.svg_name('one-line-wide'), [('&amp;', 'Compose')])],
       })
     return ftn
 
   def create_window(self):
     """Create the main window."""
-    self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
+    self.window = Gtk.Window()
+    self.window.set_name("key-mon")
+
+    rgba = self.window.get_screen().get_rgba_visual()
+    if rgba is not None:
+      self.window.set_visual(rgba)
+    self.provider = Gtk.CssProvider()
+
+    self.provider.load_from_data(
+        b"""
+    #key-mon {
+        background-color:rgba(0,0,0,0);
+    }
+        """
+    )
+    context = self.window.get_style_context()
+    context.add_provider(self.provider, Gtk.STYLE_PROVIDER_PRIORITY_APPLICATION)
+
+
+
     self.window.set_resizable(False)
 
     self.window.set_title('Keyboard Status Monitor')
@@ -283,23 +310,25 @@ class KeyMon:
 
     self.mouse_indicator_win = shaped_window.ShapedWindow(
         self.svg_name('mouse-indicator'),
+        self.options.click_opacity,
+        color=self.options.click_color,
         timeout=self.options.visible_click_timeout)
 
     self.mouse_follower_win = shaped_window.ShapedWindow(
-        self.svg_name('mouse-follower'))
+        self.svg_name('mouse-follower'), 0.5)
     if self.options.follow_mouse:
-        self.mouse_follower_win.show()
+      self.mouse_follower_win.show()
 
     self.window.set_opacity(self.options.opacity)
     self.window.set_keep_above(True)
 
-    self.event_box = gtk.EventBox()
+    self.event_box = Gtk.EventBox()
     self.window.add(self.event_box)
     self.event_box.show()
 
     self.create_images()
 
-    self.hbox = gtk.HBox(False, 0)
+    self.hbox = Gtk.Box(False, 0)
     self.event_box.add(self.hbox)
 
     self.layout_boxes()
@@ -316,58 +345,34 @@ class KeyMon:
       self.window.move(old_x, old_y)
     self.window.show()
 
-  def update_shape_mask(self, *unused_args, **kwargs):
-    if not self.options.backgroundless:
-      return
-    force = kwargs.get('force', False)
-
-    btns = [btn for btn in self.buttons if btn.get_visible()]
-    # Generate id to see if current mask needs to be updated, which is a tuple
-    # of allocation of buttons.
-    cache_id = tuple(tuple(btn.get_allocation()) for btn in btns)
-    if cache_id == self.shape_mask_current and not force:
-      return
-
-    # Try to find existing mask in cache
-    # TODO limit number of cached masks
-    shape_mask = self.shape_mask_cache.get(cache_id, None)
-    if shape_mask and not force:
-      self.window.shape_combine_mask(shape_mask, 0, 0)
-      self.shape_mask_current = cache_id
-      return
-
-    _, _, width, height = self.window.get_allocation()
-    masks = [self.pixbufs.get(btn.current).render_pixmap_and_mask()[1] \
-             for btn in btns]
-    shape_mask = gtk.gdk.Pixmap(None, width, height, masks[0].get_depth())
-
-    gc = gtk.gdk.GC(shape_mask)
-    # Initialize the mask just in case masks of buttons can't fill the window,
-    # if that happens, some artifacts will be seen usually at right edge.
-    gc.set_foreground(
-        gtk.gdk.Color(pixel=0) if self.options.backgroundless else \
-        gtk.gdk.Color(pixel=1))
-    shape_mask.draw_rectangle(gc, True, 0, 0, width, height)
-
-    for btn_allocation, mask in zip(cache_id, masks):
-      # Don't create mask until every image is allocated
-      if btn_allocation[0] == -1:
-        return
-      shape_mask.draw_drawable(gc, mask, 0, 0, *btn_allocation)
-
-    self.window.shape_combine_mask(shape_mask, 0, 0)
-    self.shape_mask_current = cache_id
-    self.shape_mask_cache[cache_id] = shape_mask
+  def update_shape_mask(self, *unused_args, **unused_kwargs):
+    """Update the shape mask"""
+    if self.options.backgroundless:
+      self.provider.load_from_data(
+          b"""
+        #key-mon {
+            background-color:rgba(0,0,0,0);
+        }
+        """)
+    else:
+      self.provider.load_from_data(
+          b"""
+        #key-mon {
+            background-color:rgba(255,255,255,1);
+        }
+        """)
 
   def create_images(self):
+    """Create the images (buttons)"""
     self.images['MOUSE'] = two_state_image.TwoStateImage(self.pixbufs, 'MOUSE')
-    for img in self.MODS:
+    for img in self.mod_constants:
       self.images[img] = two_state_image.TwoStateImage(
           self.pixbufs, img + '_EMPTY', self.enabled[img])
     self.create_buttons()
 
   def create_buttons(self):
-    self.buttons = list(self.images[img] for img in self.IMAGES)
+    """Create the buttons"""
+    self.buttons = list(self.images[img] for img in self.images_constants)
     for _ in range(self.options.old_keys):
       key_image = two_state_image.TwoStateImage(self.pixbufs, 'KEY_EMPTY')
       self.buttons.append(key_image)
@@ -381,9 +386,10 @@ class KeyMon:
       but.connect('size_allocate', self.update_shape_mask)
 
   def layout_boxes(self):
+    """Layout the buttons in the boxes"""
     for child in self.hbox.get_children():
       self.hbox.remove(child)
-    for img in self.IMAGES:
+    for img in self.images_constants:
       if not self.enabled[img]:
         self.images[img].hide()
       self.hbox.pack_start(self.images[img], False, False, 0)
@@ -405,10 +411,10 @@ class KeyMon:
   def svg_name(self, fname):
     """Return an svg filename given the theme, system."""
     themepath = self.options.themes[self.options.theme][1]
-    fullname = os.path.join(themepath, '%s%s.svg' % (fname, self.svg_size))
+    fullname = os.path.join(themepath, f'{fname}{self.svg_size}.svg')
     if self.svg_size and not os.path.exists(fullname):
       # Small not found, defaulting to large size
-      fullname = os.path.join(themepath, '%s.svg' % fname)
+      fullname = os.path.join(themepath, f'{fname}.svg')
     return fullname
 
   def add_events(self):
@@ -419,19 +425,19 @@ class KeyMon:
     self.window.connect('leave-notify-event', self.pointer_leave)
     self.event_box.connect('button_release_event', self.right_click_handler)
 
-    accelgroup = gtk.AccelGroup()
-    key, modifier = gtk.accelerator_parse('<Control>q')
-    accelgroup.connect_group(key, modifier, gtk.ACCEL_VISIBLE, self.quit_program)
+    accelgroup = Gtk.AccelGroup()
+    key, modifier = Gtk.accelerator_parse('<Control>q')
+    accelgroup.connect(key, modifier, Gtk.AccelFlags.VISIBLE, self.quit_program)
 
-    key, modifier = gtk.accelerator_parse('<Control>s')
-    accelgroup.connect_group(key, modifier, gtk.ACCEL_VISIBLE, self.show_settings_dlg)
+    key, modifier = Gtk.accelerator_parse('<Control>s')
+    accelgroup.connect(key, modifier, Gtk.AccelFlags.VISIBLE, self.show_settings_dlg)
     self.window.add_accel_group(accelgroup)
 
     if self.options.screenshot:
-      gobject.timeout_add(700, self.do_screenshot)
+      GLib.timeout_add(700, self.do_screenshot)
       return
 
-    gobject.idle_add(self.on_idle)
+    GLib.idle_add(self.on_idle)
 
   def button_released(self, unused_widget, evt):
     """A mouse button was released."""
@@ -447,16 +453,16 @@ class KeyMon:
       self.window.set_opacity(self.options.opacity)
       # remove no_press_timer
       if self.no_press_timer:
-        gobject.source_remove(self.no_press_timer)
+        GLib.source_remove(self.no_press_timer)
         self.no_press_timer = None
     return True
 
   def pointer_leave(self, unused_widget, unused_evt):
-
+    """The pointer has left window"""
     self.set_accept_focus(False)
 
   def set_accept_focus(self, accept_focus=True):
-
+    """Allow the window accepct focus or not"""
     self.window.set_accept_focus(accept_focus)
     if accept_focus:
       logging.debug('window now accepts focus')
@@ -473,16 +479,17 @@ class KeyMon:
     x, y = x + new_p[0] - old_p[0], y + new_p[1] - old_p[1]
     self.window.move(x, y)
 
-    logging.info('Moved window to %d, %d' % (x, y))
+    logging.info('Moved window to %d, %d', x, y)
     self.options.x_pos = x
     self.options.y_pos = y
 
   def on_idle(self):
     """Check for events on idle."""
-    event = self.devices.next_event()
     try:
-      if event:
-        self.handle_event(event)
+      events = list(self.next_events())
+      if events:
+        for event in events:
+          self.handle_event(event)
       else:
         for button in self.buttons:
           button.empty_event()
@@ -492,6 +499,22 @@ class KeyMon:
       return False
     return True  # continue calling
 
+  def next_events(self):
+    """Yields the next events with a single move event at the end, if any."""
+    move_event = None
+    while True:
+      event = self.devices.next_event()
+      if not event:
+        break
+      if event.type == 'EV_MOV':
+        # Ignore the previous outdated move event.
+        move_event = event
+        continue
+      yield event
+
+    if move_event:
+      yield move_event
+
   def handle_event(self, event):
     """Handle an X event."""
     if event.type == 'EV_MOV':
@@ -525,9 +548,10 @@ class KeyMon:
       self.window.show()
     self.window.set_opacity(self.options.opacity)
     if self.no_press_timer:
-      gobject.source_remove(self.no_press_timer)
+      GLib.source_remove(self.no_press_timer)
       self.no_press_timer = None
-    self.no_press_timer = gobject.timeout_add(int(self.options.no_press_fadeout * 1000), self.no_press_fadeout)
+    self.no_press_timer = GLib.timeout_add(
+        int(self.options.no_press_fadeout * 1000), self.no_press_fadeout)
 
   def no_press_fadeout(self, begin=True):
     """Fadeout the window in a second
@@ -536,8 +560,8 @@ class KeyMon:
     """
     opacity = self.window.get_opacity() - self.options.opacity / 10.0
     if opacity < 0.0:
-      opacity = 0.0;
-    logging.debug('Set opacity = %f' % opacity)
+      opacity = 0.0
+    logging.debug('Set opacity = %f', opacity)
     self.window.set_opacity(opacity)
     if opacity == 0.0:
       self.window.hide()
@@ -547,10 +571,11 @@ class KeyMon:
 
     if begin:
       # Recreate a new timer with 0.1 seccond interval
-      self.no_press_timer = gobject.timeout_add(100, self.no_press_fadeout)
+      self.no_press_timer = GLib.timeout_add(100, self.no_press_fadeout)
       # The current self.options.no_press_fadeout interval will not be timed
       # out again.
       return False
+    return True
 
   def _show_down_key(self, name):
     """Show the down key.
@@ -564,7 +589,7 @@ class KeyMon:
       return True
     if self.is_shift_code(name):
       return True
-    if (any(self.images[img].is_pressed() for img in self.MODS)):
+    if any(self.images[img].is_pressed() for img in self.mod_constants):
       return True
     return False
 
@@ -573,7 +598,7 @@ class KeyMon:
     image.really_pressed = code == 1
     if code == 1:
       if self._show_down_key(name):
-        logging.debug('Switch to %s, code %s' % (name, code))
+        logging.debug('Switch to %s, code %d', name, code)
         image.switch_to(name)
       return
 
@@ -583,12 +608,13 @@ class KeyMon:
       if not self.options.sticky_mode:
         image.switch_to_default()
       return
-    else:
-      for img in self.MODS:
-        self.images[img].reset_time_if_pressed()
-      image.switch_to_default()
+
+    for img in self.mod_constants:
+      self.images[img].reset_time_if_pressed()
+    image.switch_to_default()
 
   def is_shift_code(self, code):
+    """Is the code a shift code (like Alt)"""
     if code in ('SHIFT', 'ALT', 'ALTGR', 'CTRL', 'META'):
       return True
     return False
@@ -598,12 +624,11 @@ class KeyMon:
     code, medium_name, short_name = self.modmap.get_and_check(scan_code,
                                                               xlib_name)
     if not code:
-      logging.info('No mapping for scan_code %s', scan_code)
+      logging.info('No mapping for scan_code %d', scan_code)
       return
     if self.options.scale < 1.0 and short_name:
       medium_name = short_name
-    logging.debug('Scan code %s, Key %s pressed = %r', scan_code,
-                                                       code, medium_name)
+    logging.debug('Scan code %d, Key %d pressed = %s', scan_code, code, medium_name)
     if code in self.name_fnames:
       self._handle_event(self.key_image, code, value)
       return
@@ -652,8 +677,8 @@ class KeyMon:
             n_code = i
           if btn == self.images['MOUSE'].current:
             n_image = i
-        if self.options.emulate_middle and ((self.images['MOUSE'].current == 'BTN_LEFT'
-            and code == 'BTN_RIGHT') or
+        if self.options.emulate_middle and (
+            (self.images['MOUSE'].current == 'BTN_LEFT' and code == 'BTN_RIGHT') or
             (self.images['MOUSE'].current == 'BTN_RIGHT' and code == 'BTN_LEFT')):
           code = 'BTN_MIDDLE'
         elif value == 0 and n_code != n_image:
@@ -663,7 +688,8 @@ class KeyMon:
       elif code not in self.name_fnames:
         btn_num = code.replace('BTN_', '')
         self.name_fnames[code] = [
-            fix_svg_key_closure(self.svg_name('mouse'),
+            fix_svg_key_closure(
+                self.svg_name('mouse'),
                 [('>&#8203;', '>' + btn_num)])]
       self._handle_event(self.images['MOUSE'], code, value)
 
@@ -688,7 +714,7 @@ class KeyMon:
     elif direction < 0:
       self._handle_event(self.images['MOUSE'], 'SCROLL_DOWN', 1)
     self.images['MOUSE'].switch_to_default()
-    return True
+    return
 
   def quit_program(self, *unused_args):
     """Quit the program."""
@@ -699,7 +725,7 @@ class KeyMon:
     """Also quit the program."""
     self.devices.stop_listening()
     self.options.save()
-    gtk.main_quit()
+    Gtk.main_quit()
 
   def right_click_handler(self, unused_widget, event):
     """Handle the right click button and show a menu."""
@@ -709,30 +735,30 @@ class KeyMon:
     menu = self.create_context_menu()
 
     menu.show()
-    menu.popup(None, None, None, event.button, event.time)
+    menu.popup(None, None, None, None, event.button, event.time)
 
   def create_context_menu(self):
     """Create a context menu on right click."""
-    menu = gtk.Menu()
+    menu = Gtk.Menu()
 
-    toggle_chrome = gtk.CheckMenuItem(_('Window _Chrome'))
+    toggle_chrome = Gtk.CheckMenuItem().new_with_mnemonic(_('Window _Chrome'))
     toggle_chrome.set_active(self.window.get_decorated())
-    toggle_chrome.connect_object('activate', self.toggle_chrome,
-       self.window.get_decorated())
+    toggle_chrome.connect_object(
+        'activate', self.toggle_chrome, self.window.get_decorated())
     toggle_chrome.show()
     menu.append(toggle_chrome)
 
-    settings_click = gtk.MenuItem(_('_Settings...\tCtrl-S'))
+    settings_click = Gtk.MenuItem().new_with_mnemonic(_('_Settings...\tCtrl-S'))
     settings_click.connect_object('activate', self.show_settings_dlg, None)
     settings_click.show()
     menu.append(settings_click)
 
-    about_click = gtk.MenuItem(_('_About...'))
+    about_click = Gtk.MenuItem().new_with_mnemonic(_('_About...'))
     about_click.connect_object('activate', self.show_about_dlg, None)
     about_click.show()
     menu.append(about_click)
 
-    quitcmd = gtk.MenuItem(_('_Quit\tCtrl-Q'))
+    quitcmd = Gtk.MenuItem().new_with_mnemonic(_('_Quit\tCtrl-Q'))
     quitcmd.connect_object('activate', self.destroy, None)
     quitcmd.show()
 
@@ -754,7 +780,7 @@ class KeyMon:
 
   def settings_changed(self, unused_dlg):
     """Event received from the settings dialog."""
-    for img in self.IMAGES:
+    for img in self.images_constants:
       self._toggle_a_key(self.images[img], img, self.get_option(cstrf(img.lower)))
     self.create_buttons()
     self.layout_boxes()
@@ -786,7 +812,7 @@ class KeyMon:
 
     # reload keymap
     self.modmap = mod_mapper.safely_read_mod_map(
-            self.options.kbd_file, self.options.kbd_files)
+        self.options.kbd_file, self.options.kbd_files)
 
   def _toggle_a_key(self, image, name, show):
     """Toggle show/hide a key."""
@@ -802,23 +828,25 @@ class KeyMon:
       image.hide()
 
   def show_about_dlg(self, *_):
+    """Show the about dialog"""
 
-    dlg = gtk.AboutDialog()
+    dlg = Gtk.AboutDialog()
     # Find the logo file
     logo_paths = (os.path.join(self.pathname, '../../icons'),)
-    logo_paths += tuple(logo_path + '/share/pixmaps' for logo_path in (
+    logo_paths += tuple(
+        logo_path + '/share/pixmaps' for logo_path in (
             os.path.expanduser('~'),
             '/usr', '/usr/local', '/opt/local',
-            ))
+        ))
     logo_paths = [logo_path + '/key-mon.xpm' for logo_path in logo_paths]
     for logo_path in logo_paths:
       if os.path.exists(logo_path):
-        dlg.set_logo(gtk.gdk.pixbuf_new_from_file(logo_path))
+        dlg.set_logo(GdkPixbuf.Pixbuf.new_from_file(logo_path))
         break
 
     dlg.set_name('Keyboard Status Monitor')
     dlg.set_program_name('key-mon')
-    dlg.set_website('http://code.google.com/p/key-mon/')
+    dlg.set_website('https://github.com/scottkirkwood/key-mon')
     dlg.set_version(__version__)
     dlg.set_authors([
         __author__,
@@ -842,10 +870,11 @@ limitations under the License.''')
 
 def show_version():
   """Show the version number and author, used by help2man."""
-  print _('Keymon version %s.') % __version__
-  print _('Written by %s') % __author__
+  print(_(f'Keymon version {__version__}.'))
+  print(_(f'Written by {__author__}'))
 
 def create_options():
+  """Create the options available"""
   opts = options.Options()
 
   opts.add_option(opt_short='-s', opt_long='--smaller', dest='smaller', default=False,
@@ -913,6 +942,14 @@ def create_options():
                   ini_group='ui', ini_name='visible-click',
                   default=False,
                   help=_('Show where you clicked'))
+  opts.add_option(opt_long='--click_color', dest='click_color', type='str',
+                  ini_group='ui', ini_name='click-color',
+                  default='ff0000',
+                  help=_('The color of the click indicator'))
+  opts.add_option(opt_long='--click_opacity', dest='click_opacity', type='float',
+                  ini_group='ui', ini_name='click-opacity',
+                  default=0.5,
+                  help=_('The opacity of the click indicator'))
   opts.add_option(opt_long='--follow_mouse', dest='follow_mouse', type='bool',
                   ini_group='ui', ini_name='follow-mouse',
                   default=False,
@@ -982,14 +1019,13 @@ def main():
         loglevel = sys.argv[idx + 1]
       level = getattr(logging, loglevel.upper(), None)
       if level is None:
-          raise ValueError('Invalid log level: %s' % loglevel)
+        raise ValueError(f'Invalid log level: {loglevel}')
       loglevel = level
-  else:
-    if '--debug' in sys.argv or '-d' in sys.argv:
+    elif '--debug' in sys.argv or '-d' in sys.argv:
       loglevel = logging.DEBUG
   logging.basicConfig(
-      level=loglevel,
-      format='%(filename)s [%(lineno)d]: %(levelname)s %(message)s')
+      level=loglevel, style='{',
+      format='{filename} [{lineno}]: {levelname} {message}')
   if loglevel is None:
     # Disabling warning, info, debug messages
     logging.disable(logging.WARNING)
@@ -1009,29 +1045,30 @@ def main():
 
   opts.themes = settings.get_themes()
   if opts.list_themes:
-    print _('Available themes:')
-    print
+    print(_('Available themes:'))
+    print()
     theme_names = sorted(opts.themes)
     name_len = max(len(name) for name in theme_names)
     for theme in theme_names:
-      print (' - %%-%ds: %%s' % name_len) % (theme, opts.themes[theme][0])
+      print(f' - {theme:<{name_len}}: {opts.themes[theme][0]}')
     raise SystemExit()
-  elif opts.theme and opts.theme not in opts.themes:
-    print _('Theme %r does not exist') % opts.theme
-    print
-    print _('Please make sure %r can be found in '
-            'one of the following directories:') % opts.theme
-    print
+
+  if opts.theme and opts.theme not in opts.themes:
+    print(_(f'Theme {opts.theme!r} does not exist'))
+    print()
+    print(_(f'Please make sure {opts.theme!r} can be found in '
+            'one of the following directories:'))
+    print()
     for theme_dir in settings.get_config_dirs('themes'):
-      print ' - %s' % theme_dir
+      print(f' - {theme_dir}')
     sys.exit(-1)
   if opts.reset:
-    print _('Resetting to defaults.')
+    print(_('Resetting to defaults.'))
     opts.reset_to_defaults()
     opts.save()
   keymon = KeyMon(opts)
   try:
-    gtk.main()
+    Gtk.main()
   except KeyboardInterrupt:
     keymon.quit_program()
 
